package com.persagy.communication.mina.queue;


import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

/***
 * https://www.cnblogs.com/crazymakercircle/p/13934429.html
 * 2.1BlockingQueue的核心方法
 * 1.放入数据
 * （1）offer(anObject):表示如果可能的话,将anObject加到BlockingQueue里,即如果BlockingQueue可以容纳,则返回true,否则返回false.（本方法不阻塞当前执行方法的线程）；　　　　　　
 * （2）offer(E o, long timeout, TimeUnit unit)：可以设定等待的时间，如果在指定的时间内，还不能往队列中加入BlockingQueue，则返回失败。
 * （3）put(anObject):把anObject加到BlockingQueue里,如果BlockQueue没有空间,则调用此方法的线程被阻断直到BlockingQueue里面有空间再继续.
 *
 * 2 获取数据
 * （1）poll(time):取走BlockingQueue里排在首位的对象,若不能立即取出,则可以等time参数规定的时间,取不到时返回null;
 * （2）poll(long timeout, TimeUnit unit)：从BlockingQueue取出一个队首的对象，如果在指定时间内，队列一旦有数据可取，则立即返回队列中的数据。否则直到时间超时还没有数据可取，返回失败。
 * （3）take():取走BlockingQueue里排在首位的对象,若BlockingQueue为空,阻断进入等待状态直到BlockingQueue有新的数据被加入;
 * （4）drainTo():一次性从BlockingQueue获取所有可用的数据对象（还可以指定获取数据的个数），通过该方法，可以提升获取数据效率；不需要多次分批加锁或释放锁。
 *
 * @author lgy
 * @date 2022/4/15 18:00
 */
@Slf4j
public class MsgQueue<T> {
    //队列大小
    private int queueMaxSize = Integer.MAX_VALUE;


    private final BlockingQueue<T> blockingQueue = new LinkedBlockingQueue<>(queueMaxSize);

    public MsgQueue() {
    }

    public MsgQueue(int queueMaxSize) {
        this.queueMaxSize=queueMaxSize;
    }
    //生产入队
    public void put(T msg){
        try {
            blockingQueue.put(msg);
        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }
    }

    //消费出队
    public  T consume() throws InterruptedException {
        return blockingQueue.take();
    }
    //消费出列，非阻塞方法
    public int drainTo(Collection<? super T> c, int maxElements) throws InterruptedException {
        return blockingQueue.drainTo(c, maxElements);
    }
    //消费出列，阻塞方法
    public int drainToBlocking(Collection<? super T> c, int maxElements) throws InterruptedException {
        int count = blockingQueue.drainTo(c, maxElements);
        if(count >0){
            return count;
        }
        c.add(blockingQueue.take());
        return 1;
    }
    //消费出列，阻塞方法
    public List<T> drainToBlocking(int maxElements) throws InterruptedException {
        List<T> c = new ArrayList<>();
        int count = blockingQueue.drainTo(c, maxElements);
        if(count >0){
            return c;
        }
        c.add(blockingQueue.take());
        return c;
    }
    // 获取队列大小
    public int size() {
        return blockingQueue.size();
    }

    public BlockingQueue<T> getBlockingQueue() {
        return blockingQueue;
    }

}
